"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.plugin = exports.defineCE = void 0;
const cli_shared_utils_1 = require("@vue/cli-shared-utils");
const webpack_1 = __importDefault(require("webpack"));
// eslint-disable-next-line @typescript-eslint/prefer-ts-expect-error
// @ts-ignore
const package_json_1 = __importDefault(require("../package.json"));
const utils_1 = require("./utils");
const PLUGIN_NAME = package_json_1.default.name;
/**
 * 配置参数
 * @param options
 * @returns
 */
const defineCE = (options) => {
    return { [PLUGIN_NAME]: options };
};
exports.defineCE = defineCE;
/**
 * vue template 自定义元素编译优化插件
 *
 */
const plugin = (api, options) => {
    var _a;
    const vue = (0, cli_shared_utils_1.loadModule)('vue', api.service.context);
    const isVue3 = vue && cli_shared_utils_1.semver.major(vue.version) === 3;
    const pluginOptions = ((_a = options.pluginOptions) === null || _a === void 0 ? void 0 : _a[PLUGIN_NAME]) || {};
    // thread-loader 不支持传递不可序列化的参数
    if (options.parallel && process.env.NODE_ENV === 'production') {
        throw new Error(`[${PLUGIN_NAME}] 开启 parallel 将导致编译失败, 请关闭`);
    }
    if (typeof pluginOptions.customElement === 'undefined') {
        throw new Error(`[${PLUGIN_NAME}] 必须设置 customElement 参数:
\`\`\`
// vue.config.js
const { defineCE } = require('@wakeadmin/vue-cli-plugin-ce');

module.exports = defineConfig({
  pluginOptions: {
    ...defineCE({customElement: /wkc-/})
  }
})
\`\`\`
    `);
    }
    const matcher = (0, utils_1.createMatcher)(pluginOptions);
    api.chainWebpack(chain => {
        // 注入配置参数
        chain
            .plugin('ceDefine')
            .use(webpack_1.default.DefinePlugin, [
            {
                // 配置 @wakeadmin/ce 插件使用
                __CE_OPTIONS__: (0, utils_1.stringifyOptions)(pluginOptions),
            },
        ])
            .end();
        if (isVue3) {
            const { nodeTransform } = require('./vue3');
            chain.module
                .rule('vue')
                .use('vue-loader')
                .tap(opts => (Object.assign(Object.assign({}, opts), { compilerOptions: Object.assign(Object.assign({}, opts.compilerOptions), { 
                    // treat any tag that starts with ion- as custom elements
                    isCustomElement: matcher.isCustomElement, nodeTransforms: [nodeTransform(pluginOptions)] }) })));
        }
        else {
            const { TransformModule } = require('./vue2');
            chain.module
                .rule('vue')
                .use('vue-loader')
                .tap(opts => (Object.assign(Object.assign({}, opts), { compilerOptions: Object.assign(Object.assign({}, opts.compilerOptions), { modules: [
                        // 在 vue 2 下，保持 web component 的使用行为和 vue3 一致
                        new TransformModule(pluginOptions),
                    ] }) })));
        }
    });
};
exports.plugin = plugin;
exports.default = exports.plugin;
