"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createMatcher = exports.stringifyOptions = void 0;
function stringifyOptions(options) {
    const stringifyRules = (rules) => {
        const arr = Array.isArray(rules) ? rules : rules ? [rules] : [];
        return `[${arr.map(i => (typeof i === 'string' ? JSON.stringify(i) : i.toString())).join(', ')}]`;
    };
    const { customElement, mustUseProp } = options;
    return `"{ customElement: ${stringifyRules(customElement)}, mustUseProp: ${stringifyRules(mustUseProp)} }"`;
}
exports.stringifyOptions = stringifyOptions;
const createMatcher = (options) => {
    const { customElement, mustUseProp } = options !== null && options !== void 0 ? options : {};
    const normalized = (rules) => {
        const arr = Array.isArray(rules) ? rules : rules ? [rules] : null;
        if (arr) {
            const strings = new Set();
            const regexp = [];
            for (const i of arr) {
                if (typeof i === 'string') {
                    strings.add(i);
                }
                else {
                    regexp.push(i);
                }
            }
            return (tag) => {
                if (strings.size && strings.has(tag)) {
                    return true;
                }
                for (const reg of regexp) {
                    if (reg.test(tag)) {
                        return true;
                    }
                }
                return false;
            };
        }
        return () => false;
    };
    const isCustomElement = normalized(customElement);
    return {
        isCustomElement: tag => {
            return typeof tag === 'string' && tag.includes('-') && isCustomElement(tag);
        },
        mustUseProp: normalized(mustUseProp),
    };
};
exports.createMatcher = createMatcher;
