"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.nodeTransform = void 0;
const compiler_core_1 = require("@vue/compiler-core");
const utils_1 = require("../utils");
// @vue/compiler-core 没有实际导出 NodeTypes(const)
const ELEMENT = 1;
const DIRECTIVE = 7;
const NOT_CONSTANT = 0;
const CAN_STRINGIFY = 3;
/**
 * 创建 vue3 的模板转换器，用于优化自定义组件的使用
 */
const nodeTransform = (options) => {
    const matcher = (0, utils_1.createMatcher)(options);
    return (node, context) => {
        var _a;
        if (node.type !== ELEMENT) {
            return;
        }
        const tag = node.tag;
        if (!matcher.isCustomElement(tag)) {
            return;
        }
        // 查找 v-model
        const vModel = node.props.find((p) => p.type === DIRECTIVE && p.name === 'model');
        if (vModel == null) {
            return;
        }
        const lazy = (_a = vModel.modifiers) === null || _a === void 0 ? void 0 : _a.includes('lazy');
        // 转换 v-model 为 input/value
        const valueProp = {
            type: DIRECTIVE,
            name: 'bind',
            loc: vModel.loc,
            modifiers: [],
            arg: (0, compiler_core_1.createSimpleExpression)('value', true, vModel.loc, CAN_STRINGIFY),
            exp: vModel.exp,
        };
        const eventHandler = (0, compiler_core_1.createSimpleExpression)(
        // 这里使用 source 避免上游转换的副作用
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        `(${vModel.exp.loc.source}) = $event.detail[0]`, false, vModel.loc, NOT_CONSTANT);
        const eventDirective = {
            type: DIRECTIVE,
            name: 'on',
            loc: vModel.loc,
            modifiers: [],
            arg: (0, compiler_core_1.createSimpleExpression)(lazy ? 'change' : 'input', true, vModel.loc, CAN_STRINGIFY),
            exp: eventHandler,
        };
        const vModelIndex = node.props.indexOf(vModel);
        node.props.splice(vModelIndex, 1, valueProp, eventDirective);
    };
};
exports.nodeTransform = nodeTransform;
